@@ -15,12 +15,13 @@ from json import JSONDecodeError
 
 from typing import get_type_hints, Iterable
 
-from .commons import raise_errs_if_needed, wrap_val, _is_sunder, _is_dunder
+from .commons import Constant, raise_errs_if_needed, wrap_val, _is_sunder, _is_dunder
 from .utility import type_is_generic
 
 REQUIRED_FIELDS = "_required"
 DEFAULTS = "_defaults"
-ADDITIONAL_PROPERTIES = "_additionalProperties"
+ADDITIONAL_PROPERTIES = "_additional_properties"
+OLD_ADDITIONAL_PROPERTIES = "_additionalProperties"
 IS_IMMUTABLE = "_immutable"
 OPTIONAL_FIELDS = "_optional"
 MUST_BE_UNIQUE = "_must_be_unique"
@@ -72,7 +73,13 @@ class ImmutableMixin:
 
 
 def make_signature(
-    names, required, additional_properties, bases_params_by_name, bases_required
+    names,
+    *,
+    required,
+    additional_properties,
+    bases_params_by_name,
+    bases_required,
+    constants,
 ):
     """
     Make a signature that will be used for the constructor of the Structure
@@ -84,10 +91,11 @@ def make_signature(
     :return: the signature
     """
     all_names = set(names) | set(bases_params_by_name.keys())
+    all_except_consts = all_names - set(constants)
     non_default_args_for_class = OrderedDict(
         [
             (name, Parameter(name, Parameter.POSITIONAL_OR_KEYWORD))
-            for name in all_names
+            for name in all_except_consts
             if name in required
         ]
     )
@@ -95,7 +103,7 @@ def make_signature(
         [
             (name, param)
             for (name, param) in bases_params_by_name.items()
-            if (name in required or name in bases_required)
+            if (name in required or name in bases_required) and name not in constants
         ]
     )
     non_default_args = list(
@@ -106,14 +114,18 @@ def make_signature(
         [
             (name, Parameter(name, Parameter.POSITIONAL_OR_KEYWORD, default=None))
             for name in names
-            if name not in required
+            if (name not in required and name not in constants)
         ]
     )
     default_args_for_bases = OrderedDict(
         [
             (name, param)
             for (name, param) in bases_params_by_name.items()
-            if (name not in required and name not in bases_required)
+            if (
+                name not in required
+                and name not in bases_required
+                and name not in constants
+            )
         ]
     )
     default_args = list({**default_args_for_bases, **default_args_for_class}.values())
@@ -149,8 +161,11 @@ def get_base_info(bases):
                 if param.default is not None and param.kind != Parameter.VAR_KEYWORD:
                     bases_required.append(k)
                 bases_params[k] = param
+        additional_props_fallback = base.__dict__.get(
+            OLD_ADDITIONAL_PROPERTIES, TypedPyDefaults.additional_properties_default
+        )
         additional_props = base.__dict__.get(
-            ADDITIONAL_PROPERTIES, TypedPyDefaults.additional_properties_default
+            ADDITIONAL_PROPERTIES, additional_props_fallback
         )
         if additional_props and bases_params["kwargs"].kind == Parameter.VAR_KEYWORD:
             del bases_params["kwargs"]
@@ -363,7 +378,7 @@ class Field(UniqueMixin, metaclass=_FieldMeta):
             if name in owner.__dict__:
                 return owner.__dict__[name]
             field_by_name = owner.get_all_fields_by_name()
-            return field_by_name[name]
+            return field_by_name.get(name)
 
         if instance is not None and self._name not in instance.__dict__:
             default_value = (
@@ -623,9 +638,14 @@ class StructMeta(type):
                 if key not in clsobj.__annotations__ and isinstance(val, TypedField):
                     clsobj.__annotations__[key] = getattr(val, "_ty")
 
-        default_required = (
-            list(set(bases_required + fields)) if bases_params else fields
-        )
+        all_fields = set(bases_required + fields) if bases_params else fields
+        default_required = list(all_fields)
+
+        clsobj._constants = {}
+        for fname, fval in _get_all_fields_by_name(clsobj).items():
+            if isinstance(getattr(clsobj, fname), Constant):
+                clsobj._constants[fname] = getattr(clsobj, fname)._val
+
         required = cls_dict.get(REQUIRED_FIELDS, default_required)
         setattr(clsobj, REQUIRED_FIELDS, list(set(bases_required + required)))
         optional_fields = cls_dict.get(OPTIONAL_FIELDS, [])
@@ -635,16 +655,21 @@ class StructMeta(type):
                     "optional cannot override prior required in the class or in a base class"
                 )
 
+        if OLD_ADDITIONAL_PROPERTIES in cls_dict:
+            cls_dict[ADDITIONAL_PROPERTIES] = cls_dict[OLD_ADDITIONAL_PROPERTIES]
+            setattr(clsobj, ADDITIONAL_PROPERTIES, cls_dict[ADDITIONAL_PROPERTIES])
+            delattr(clsobj, OLD_ADDITIONAL_PROPERTIES)
         additional_props = cls_dict.get(
             ADDITIONAL_PROPERTIES, TypedPyDefaults.additional_properties_default
         )
 
         sig = make_signature(
             clsobj._fields,
-            required,
-            additional_props,
-            bases_params,
+            required=required,
+            additional_properties=additional_props,
+            bases_params_by_name=bases_params,
             bases_required=bases_required,
+            constants=clsobj._constants.keys(),
         )
         setattr(clsobj, "__signature__", sig)
         setattr(clsobj, "_field_by_name", _get_all_fields_by_name(clsobj))
@@ -843,15 +868,15 @@ class Structure(UniqueMixin, metaclass=StructMeta):
                     _optional = ['name']
 
 
-        _additionalProperties(bool): optional
+        _additiona_properties(bool): optional
             Is it allowed to add additional properties that are not defined in the class definition?
-            the default is True.
+            the default is True. This replaces the old _additionalProperties setting, which is still supported.
             Example:
 
             .. code-block:: python
 
                 class Foo(Structure):
-                    _additionalProperties = False
+                    _additional_properties = False
 
                     id = Integer
 
@@ -914,6 +939,14 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             if getattr(value, "_default", None) is not None
             and key not in bound.arguments
         ]
+
+        for field_name, const_val in self._constants.items():
+            if field_name in kwargs:
+                raise ValueError(
+                    f"{self.__class__.__name__}:  {field_name} is defined as a constant. It cannot be set."
+                )
+            setattr(self, field_name, const_val)
+
         for field_name in defaults_fields:
             default = getattr(field_by_name[field_name], "_default")
             default_value = default() if callable(default) else default
@@ -968,6 +1001,10 @@ class Structure(UniqueMixin, metaclass=StructMeta):
                 )
                 value = deepcopy(value) if needs_defensive_copy else value
 
+        if key in self._constants and getattr(self, "_instantiated", False):
+            raise ValueError(
+                f"{self.__class__.__name__}:  {key} is defined as a constant. It cannot be set."
+            )
         if not any(
             [
                 getattr(
